'use strict'

const assert = require('assert')
const abemed = require('../index')

describe('abemed - no options', function () {
  const abem = abemed()
  assert.strictEqual(abem('block'), 'block')
  assert.strictEqual(abem('block', 'elem'), 'block__elem')
  assert.strictEqual(abem('block', 'elem', 'mod'), 'block__elem -mod')
  assert.strictEqual(abem('block', 'elem', ['mod1', 'mod2']), 'block__elem -mod1 -mod2')
  assert.strictEqual(abem('block', 'elem', { mod1: true, mod2: true }), 'block__elem -mod1 -mod2')
  assert.strictEqual(abem('block', 'elem', { mod1: false, mod2: true }), 'block__elem -mod2')
  assert.strictEqual(abem('block', 'elem', { mod1: false, mod2: false }, 'mix'), 'block__elem mix')
  assert.strictEqual(abem('block', 'elem', { mod1: false, mod2: false }, ['mix1', 'mix2']), 'block__elem mix1 mix2')
  assert.strictEqual(
    abem('block', 'elem', { mod1: false, mod2: false }, { mix1: true, mix2: false }),
    'block__elem mix1'
  )
  assert.strictEqual(abem('block', null, { mod1: false, mod2: true }, ['mix1', 'mix2']), 'block -mod2 mix1 mix2')
})

describe('abemed - with options', function () {
  const abem = abemed(separators: {
          element: '___',
          modifier: '--',
        })
  assert.strictEqual(abem('block'), 'block')
  assert.strictEqual(abem('block', 'elem'), 'block__elem')
  assert.strictEqual(abem('block', 'elem', 'mod'), 'block__elem -mod')
  assert.strictEqual(abem('block', 'elem', ['mod1', 'mod2']), 'block__elem -mod1 -mod2')
  assert.strictEqual(abem('block', 'elem', { mod1: true, mod2: true }), 'block__elem -mod1 -mod2')
  assert.strictEqual(abem('block', 'elem', { mod1: false, mod2: true }), 'block__elem -mod2')
  assert.strictEqual(abem('block', 'elem', { mod1: false, mod2: false }, 'mix'), 'block__elem mix')
  assert.strictEqual(abem('block', 'elem', { mod1: false, mod2: false }, ['mix1', 'mix2']), 'block__elem mix1 mix2')
  assert.strictEqual(
    abem('block', 'elem', { mod1: false, mod2: false }, { mix1: true, mix2: false }),
    'block__elem mix1'
  )
  assert.strictEqual(abem('block', null, { mod1: false, mod2: true }, ['mix1', 'mix2']), 'block -mod2 mix1 mix2')
})
