'use strict'

module.exports = opts => {
  opts = opts || {}
  const separators = opts.separators || {}

  const capitalize = str => str.replace(/(?:^|\s)\S/g, a => a.toUpperCase())

  const abe = (sep => (block, element) => (element ? block + sep + element : block))(separators.element || '__')
  const abem = (sep => (element, modifier) => element + sep + modifier)(separators.modifier || '--')
  const abemv = (sep => (element, modifier, value) => abem(element, modifier) + sep + value)(separators.value || '-')

  const generate = (block, element, modifiers, mixins) => {
    element = abe(block, element)
    let className = element

    switch (typeof modifiers) {
      case 'string':
        className += ' ' + abem(element, modifiers)
        break
      case 'object':
        if (Array.isArray(modifiers)) {
          for (let i = 0; i < modifiers.length; i++) {
            className += ' ' + abem(element, modifiers[i])
          }
          break
        }
        for (let modifier in modifiers) {
          let value = modifiers[modifier]
          if (value === false) {
            continue
          }
          className += ' ' + abem(element, modifier)
          if (value !== true) {
            className += ' ' + abemv(element, modifier, value)
          }
        }
        break
    }

    switch (typeof mixins) {
      case 'string':
        className += ' ' + mixins
        break
      case 'object':
        if (Array.isArray(mixins)) {
          for (let i = 0; i < mixins.length; i++) {
            className += ' ' + mixins[i]
          }
          break
        }
        for (let mixin in mixins) {
          if (mixins[mixin]) {
            className += ' ' + mixin
          }
        }
        break
    }
    return className
  }

  return { generate, helpers: { be: abe, bem: abem, bemv: abemv } }
}
