'use strict'

module.exports = opts => {
  opts = opts || {}
  const separators = opts.separators || {}

  const camelize = str =>
    str.replace(/(?:^|\s)\S/g, (word, index) => (index === 0 ? word.toLowerCase() : word.slice(1).toUpperCase()))

  const elementSeparator = separators.element || '__'

  const abem = (sep => (_, modifier) => element + sep + modifier)(separators.modifier || '--')
  const abemv = (sep => (element, modifier, value) => abem(element, modifier + capitalize(value)))(
    separators.value || '-',
  )

  const generate = (block, element, modifiers, mixins) => {
    let className = element ? block + elementSeparator + element : block

    if (typeof modifiers === 'string') {
      className += ' -' + modifiers
    } else if (Array.isArray(modifiers)) {
      for (let i = 0, len = modifiers.length; i < len; ++i) className += ' -' + modifiers[i]
    } else {
      for (let modifier in modifiers) {
        let value = modifiers[modifier]
        if (value === false) {
          continue
        }
        className += ' ' + abem(element, modifier)
        if (value !== true) {
          className += ' ' + abemv(element, modifier, value)
        }
      }
    }

    switch (typeof modifiers) {
      case 'string':
        className += ' -' + modifiers
        break
      case 'object':
        if (Array.isArray(modifiers)) {
          for (let i = 0, len = modifiers.length; i < len; ++i) {
            className += ' -' + modifiers[i]
          }
          break
        }
        for (let modifier in modifiers) {
          let value = modifiers[modifier]
          if (value === false) {
            continue
          }
          className += ' ' + abem(element, modifier)
          if (value !== true) {
            className += ' ' + abemv(element, modifier, value)
          }
        }
        break
    }

    switch (typeof mixins) {
      case 'string':
        className += ' ' + mixins
        break
      case 'object':
        if (Array.isArray(mixins)) {
          for (let i = 0; i < mixins.length; i++) {
            className += ' ' + mixins[i]
          }
          break
        }
        for (let mixin in mixins) {
          if (mixins[mixin]) {
            className += ' ' + mixin
          }
        }
        break
    }
    return className
  }

  return { generate, helpers: { be: abe, bem: abem, bemv: abemv } }
}
